@@ -49,7 +49,7 @@ char copyright[] =
  * From: @(#)rlogind.c	5.53 (Berkeley) 4/20/91
  */
 char rcsid[] = 
-  "$Id: rlogind.c,v 1.11 1996/07/23 08:18:58 dholland Exp $";
+  "$Id: rlogind.c,v 1.16 1996/08/16 22:28:27 dholland Exp $";
 
 /*
  * remote login server:
@@ -74,21 +74,23 @@ char rcsid[] =
 #include <netdb.h>
 
 #include <pwd.h>
+#include <grp.h>
 #include <syslog.h>
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include "pathnames.h"
+#include "logwtmp.h"
 
 #ifdef USE_PAM
-#include <grp.h>
+#include <sys/types.h>
 #include <security/pam_appl.h>
+#include <security/pam_misc.h>
 #endif
 
 pid_t forkpty(int *, char *, struct termios *, struct winsize *);
 int logout(const char *);
-void logwtmp(const char *line, const char *name, const char *host);
 
 #ifndef TIOCPKT_WINDOW
 #define TIOCPKT_WINDOW 0x80
@@ -123,16 +125,15 @@ static void usage(void);
 static int local_domain(const char *h);
 static const char *topdomain(const char *h);
 
-
+extern int _check_rhosts_file;
 
 int
 main(int argc, char **argv)
 {
-	extern int opterr, optind;
-	extern int _check_rhosts_file;
 	int ch;
 	int on = 1, fromlen;
 	struct sockaddr_in from;
+	_check_rhosts_file = 1;     /* default */
 
 	openlog("rlogind", LOG_PID | LOG_CONS, LOG_AUTH);
 
@@ -185,7 +186,6 @@ main(int argc, char **argv)
 int	netf;
 char	line[MAXPATHLEN];
 int	confirmed;
-extern	char	*inet_ntoa();
 
 struct winsize win = { 0, 0, 0, 0 };
 
@@ -310,13 +310,11 @@ doit(int f, struct sockaddr_in *fromp)
 				exit(1);
 			}
 #ifdef USE_PAM
-			pam_end(pamh, PAM_AUTH_ERR);
-/* I gather that PAM precludes dropping to straight login or something. (?) */
-#else
+			pam_end(pamh, PAM_SUCCESS);
+#endif
 			execl(_PATH_LOGIN, "login", "-p",
 			      "-h", hp->h_name, lusername, 0);
 			/* should not return... */
-#endif
 		}
 		fatal(STDERR_FILENO, _PATH_LOGIN, 1);
 		/*NOTREACHED*/
@@ -343,8 +341,9 @@ control(int pty, char *cp, int n)
 {
 	struct winsize w;
 
-	if (n < 4+sizeof (w) || cp[2] != 's' || cp[3] != 's')
-		return (0);
+	if (n < 4+(int)sizeof(w) || cp[2] != 's' || cp[3] != 's') {
+		return 0;
+	}
 	oobdata[0] &= ~TIOCPKT_WINDOW;	/* we know he heard */
 	memcpy(&w, cp+4, sizeof(w));
 	w.ws_row = ntohs(w.ws_row);
@@ -363,7 +362,7 @@ protocol(int f, int p)
 {
 	char pibuf[1024+1], fibuf[1024], *pbp = NULL, *fbp = NULL;
 	register pcc = 0, fcc = 0;
-	int cc, nfd, n;
+	int cc, nfd, m;
 	char cntl;
 
 	/*
@@ -400,12 +399,12 @@ protocol(int f, int p)
 			} else
 				FD_SET(p, &ibits);
 		FD_SET(p, &ebits);
-		if ((n = select(nfd, &ibits, omask, &ebits, 0)) < 0) {
+		if ((m = select(nfd, &ibits, omask, &ebits, 0)) < 0) {
 			if (errno == EINTR)
 				continue;
 			fatal(f, "select", 1);
 		}
-		if (n == 0) {
+		if (m == 0) {
 			/* shouldn't happen... */
 			sleep(5);
 			continue;
@@ -428,7 +427,7 @@ protocol(int f, int p)
 				fcc = 0;
 			else {
 				register char *cp;
-				int left, n;
+				int left, nn;
 
 				if (fcc <= 0)
 					break;
@@ -439,12 +438,12 @@ protocol(int f, int p)
 					if (cp[0] == magic[0] &&
 					    cp[1] == magic[1]) {
 						left = fcc - (cp-fibuf);
-						n = control(p, cp, left);
-						if (n) {
-							left -= n;
+						nn = control(p, cp, left);
+						if (nn) {
+							left -= nn;
 							if (left > 0)
-								bcopy(cp+n, cp, left);
-							fcc -= n;
+								bcopy(cp+nn, cp, left);
+							fcc -= nn;
 							goto top; /* n^2 */
 						}
 					}
@@ -502,12 +501,12 @@ static void
 cleanup(int ignore)
 {
 	char *p;
+	(void)ignore;
 
 	p = line + sizeof(_PATH_DEV) - 1;
 	if (logout(p))
 		logwtmp(p, "", "");
 #ifdef USE_PAM
-       pam_close_session(pamh, 0);
        pam_end (pamh, PAM_SUCCESS);
 #endif
 
@@ -553,7 +552,6 @@ fatal(int f, const char *msg, int syserr)
 	write(f, buf, bp + len - buf);
 
 #ifdef USE_PAM
-	pam_close_session(pamh, 0);
 	pam_end (pamh, PAM_SUCCESS);
 #endif
 
@@ -565,7 +563,10 @@ do_rlogin(const char *host)
 {
 #ifdef USE_PAM
 	char c;
-	extern struct pam_conv conv;
+	static struct pam_conv conv = {
+	  misc_conv,
+	  NULL
+	};
         int retval;
 #endif /* USE_PAM */
 
@@ -587,7 +588,7 @@ do_rlogin(const char *host)
        (void) pam_set_item (pamh, PAM_USER,  lusername);
        (void) pam_set_item (pamh, PAM_RUSER, rusername);
        (void) pam_set_item (pamh, PAM_RHOST, host);
-       (void) pam_set_item (pamh, PAM_TTY,   "/dev/tty");
+       (void) pam_set_item (pamh, PAM_TTY, "tty");
 
        c = 0;
        write (0, &c, 1);
@@ -599,11 +600,22 @@ do_rlogin(const char *host)
                        retval = pam_acct_mgmt(pamh, 0);
                if (retval == PAM_SUCCESS)
                        break;
-               if (retval == PAM_AUTHTOKEN_REQD)
+               if (retval == PAM_AUTHTOKEN_REQD) {
                        retval = pam_chauthtok (pamh,PAM_CHANGE_EXPIRED_AUTHTOK);
-       } while (retval == PAM_SUCCESS);
+	               if(retval == PAM_SUCCESS)
+			 /* Try authentication again if passwd change
+			    succeeded.  Don't try again if it didn't;
+			    sysadmin might not want passwords changed
+			    over the next, and might have set password
+			    to pam_deny.so to disable it... */
+			 continue;
+	       }
+       } while (0); /* We have the while(0) here because it is either using
+		       that and the breaks, or goto's */
+	/* eww. -dah */
 
-       if (retval == PAM_SUCCESS) {
+
+	if (retval == PAM_SUCCESS) {
                if (setgid(pwd->pw_gid) != 0) {
                        fprintf(stderr, "cannot assume gid\n");
                        return (0);
@@ -615,17 +627,21 @@ do_rlogin(const char *host)
                }
 
                retval = pam_setcred(pamh, PAM_CRED_ESTABLISH);
-       }
+	}
 
-       if (retval == PAM_SUCCESS)
-               retval = pam_open_session(pamh,0);
-       return (retval ^ PAM_SUCCESS);
-#else
+	if (retval != PAM_SUCCESS) {
+		syslog(LOG_ERR,"PAM authentication failed for in.rlogind");
+		fatal(STDERR_FILENO, "login failed", 0);
+		/* no return */
+	}
+	return 0;
+
+#else /* !USE_PAM */
 
 	if (pwd->pw_uid == 0)
 		return(-1);
 	return(ruserok(host, 0, rusername, lusername));
-#endif /* USE_PAM */
+#endif /* PAM */
 }
 
 static void
